@@ -1,8 +1,7 @@
 ;;; -*- coding: iso-2022-7bit; -*-
 ;;; tramp-util.el --- Misc utility functions to use with Tramp
 
-;; Copyright (C) 2001, 2002, 2003, 2004, 2005,
-;;   2006 Free Software Foundation, Inc.
+;; Copyright (C) 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
 ;; Author: kai.grossjohann@gmx.net
 ;; Keywords: comm, extensions, processes
@@ -19,8 +18,8 @@
 
 ;; You should have received a copy of the GNU General Public License
 ;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+;; Boston, MA 02111-1307, USA.
 
 ;;; Commentary:
 
@@ -29,24 +28,13 @@
 
 ;;; Code:
 
+(eval-when-compile (require 'cl))
 (require 'compile)
 (require 'tramp)
-(add-hook 'tramp-util-unload-hook
-	  '(lambda ()
-	     (when (featurep 'tramp)
-	       (unload-feature 'tramp 'force))))
 
 ;; Define a Tramp minor mode. It's intention is to redefine some keys for Tramp
 ;; specific functions, like compilation.
-;; The key remapping works since Emacs 22 only. Unknown for XEmacs.
-
-;; Pacify byte-compiler
-(eval-when-compile
-  (unless (fboundp 'define-minor-mode)
-      (defalias 'define-minor-mode 'identity)
-      (defvar tramp-minor-mode))
-  (unless (featurep 'xemacs)
-    (defalias 'add-menu-button 'ignore)))
+;; The key remapping works since Emacs 21.4 only. Unknown for XEmacs.
 
 (defvar tramp-minor-mode-map (make-sparse-keymap)
   "Keymap for Tramp minor mode.")
@@ -61,14 +49,7 @@
 	(and tramp-minor-mode (tramp-tramp-file-p default-directory))))
 
 (add-hook 'find-file-hooks 'tramp-minor-mode t)
-(add-hook 'tramp-util-unload-hook
-	  '(lambda ()
-	     (remove-hook 'find-file-hooks 'tramp-minor-mode)))
-
 (add-hook 'dired-mode-hook 'tramp-minor-mode t)
-(add-hook 'tramp-util-unload-hook
-	  '(lambda ()
-	     (remove-hook 'dired-mode-hook 'tramp-minor-mode)))
 
 (defun tramp-remap-command (old-command new-command)
   "Replaces bindings of OLD-COMMAND by NEW-COMMAND.
@@ -76,10 +57,10 @@ If remapping functionality for keymaps is defined, this happens for all
 bindings.  Otherwise, only bindings active during invocation are taken
 into account.  XEmacs menubar bindings are not changed by this."
   (if (functionp 'command-remapping)
-      ;; Emacs 22
+      ;; Emacs 21.4
       (eval
        `(define-key tramp-minor-mode-map [remap ,old-command] new-command))
-    ;; previous Emacs versions.
+    ;; previous Emacs 21 versions.
     (mapcar
      '(lambda (x)
 	(define-key tramp-minor-mode-map x new-command))
@@ -112,6 +93,8 @@ into account.  XEmacs menubar bindings are not changed by this."
                                    '(compile-history . 1)))
      (list compile-command)))
   (setq compile-command command)
+  (unless (string-match "[ \t]*&[ \t]*\\'" command)
+    (setq command (concat command "&")))
   (save-some-buffers (not compilation-ask-about-save) nil)
   (let ((d default-directory))
     (save-excursion
@@ -120,17 +103,22 @@ into account.  XEmacs menubar bindings are not changed by this."
       (setq default-directory d)))
   (tramp-handle-shell-command command (get-buffer "*Compilation*"))
   (pop-to-buffer (get-buffer "*Compilation*"))
-  (tramp-minor-mode 1)
-  (compilation-minor-mode 1))
+  (compilation-mode)
+  (compilation-minor-mode 1)
+  (tramp-minor-mode 1))
 
 (defun tramp-recompile ()
   "Re-compile on remote host."
   (interactive)
-  (save-some-buffers (not compilation-ask-about-save) nil)
-  (tramp-handle-shell-command compile-command (get-buffer "*Compilation*"))
-  (pop-to-buffer (get-buffer "*Compilation*"))
-  (tramp-minor-mode 1)
-  (compilation-minor-mode 1))
+  (let ((command compile-command))
+    (unless (string-match "[ \t]*&[ \t]*\\'" command)
+      (setq command (concat command "&")))
+    (save-some-buffers (not compilation-ask-about-save) nil)
+    (tramp-handle-shell-command command (get-buffer "*Compilation*"))
+    (pop-to-buffer (get-buffer "*Compilation*"))
+    (compilation-mode)
+    (compilation-minor-mode 1)
+    (tramp-minor-mode 1)))
 
 (provide 'tramp-util)
 
